/*
 * Copyright (C) 2001-2001 OSS Nokalva Inc.  All rights reserved.
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */

/*
 * This example demonstrates OSS ASN.1 Tools for Java using a simple ASN.1
 * syntax, BCAS (Baseball Card Abstract Syntax).  The code constructs
 * a PDU objects for encoding, compares two PDU objects, checks constraints,
 * prints the constructed PDU object, encodes the constructed PDU object,
 * prints the encoded PDU, decodes the encoded PDU, prints the decoded
 * PDU object, encodes the PDU object to a file and decodes from the file.
 *
 * To run the test say:
 *
 * asn1pjav baseball.asn
 * cd baseball
 * baseball.bat javac
 * cd ..
 * javac -g Tbcas.java
 * java Tbcas
 */


/* Compiler-generated classes */
import baseball.*;
import baseball.bcas.*;

/* Universal classes from the OSS runtime library */
import com.oss.asn1.*;
import com.oss.util.*;

/* Java I/O classes */
import java.io.*;

public class Tbcas {

    /**
     * Constructor.
     */
    public Tbcas () {
    }

    public static void main(String args[]) {

		// Initialize the project.
	try {
	    Baseball.initialize();
	}
	catch (Exception e) {
	    System.out.println("Initialization failed with code: " + e);
	    System.exit(1);
	}

		//
		// Construct a PDU object for encoding.
		//
	System.out.println("Constructing a BBcard PDU object with arguments...");
	BBCard myCard = new BBCard (
	    new IA5String (
		"Casey"
	    ),
	    new IA5String (
		"Mudville Nine"
	    ),
	    32,
	    new IA5String (
		"left field"
	    ),
	    BBCard.Handedness.ambidextrous,
	    250.E-3
	);

		//
		// Constructing another PDU object with the default constructor
		// and mutator methods.
		//
	System.out.println("Constructing a BBcard PDU object with the default constructor and mutator methods...");
	BBCard anotherCard = new BBCard();
		//
		// Set the value of the field "name".
		//
	anotherCard.setName(new IA5String("Casey"));
		//
		// Set the value of the field "team".
		//
	anotherCard.setTeam(new IA5String("Mudville Nine"));
		//
		// Set the value of the field "age".
		//
	anotherCard.setAge(32);
		//
		// Set the value of the field "position".
		//
	anotherCard.setPosition(new IA5String("left field"));
		//
		// Set the value of the field "handedness".
		//
	anotherCard.setHandedness(BBCard.Handedness.ambidextrous);
		//
		// Set the value of the field "batting-average".
		//
	anotherCard.setBatting_average(250.E-3);

		//
		// Compare the two unencoded PDU objects.
		//
	System.out.println("Comparing the two PDU objects...");
	if (myCard.equalTo(anotherCard))
	    System.out.println("Comparison succeeded.");
	else
	    System.out.println("Comparison failed.");

		//
		// Check constraints.
		//
	try {
	    System.out.println("Checking constraints...");
	    Coder coder = Baseball.getBERCoder();
	    final int success = 0;
            if (myCard.isValid())
		System.out.println("Constraint checking succeeded.");
	}
	catch (ValidateFailedException e) {
	    System.out.println("Constraint checking failed with error code: " + e);
	}
	catch (ValidateNotSupportedException e) {
	    System.out.println("Constraint checking failed with error code: " + e);
	}

		//
		// Encode the constructed BBcard PDU object using BER.
		//
	try {
	    Coder coder = Baseball.getBERCoder();
	    ByteArrayOutputStream sink = new ByteArrayOutputStream();

		//
		// Enable tracing informationt from the encoder and the decoder.
		//
	    coder.enableEncoderDebugging();
	    coder.enableDecoderDebugging();

		//
		// Print the unencoded PDU object.
		//
	    System.out.println("\nUnencoded PDU object...\n");
	    System.out.println(myCard);

		//
		// Encode the PDU object.
		//
	    System.out.println("\nThe encoder's trace messages...");
	    coder.encode(myCard, sink);

		//
		// Extract the encoded PDU from the sink stream.
		//
	    byte[] encoding = sink.toByteArray();

		//
		// Print the encoded PDU.
		//
	    System.out.println("Printing the encoded PDU of " + encoding.length + " bytes...");
	    HexTool.printHex(encoding);

	    try {
		ByteArrayInputStream source = new ByteArrayInputStream(encoding);

			//
			// Decode the encoded PDU whose encoding is in the
			// "encoding" byte array.
			//
		System.out.println("\nThe decoder's trace messages...\n");
		BBCard decodedCard = (BBCard)coder.decode(source, new BBCard());
		System.out.println("PDU decoded.");
			//
			// Print out the player's batting average.
			//
		double batting_average = decodedCard.getBatting_average();
		String name = decodedCard.getName().stringValue();
		String team = decodedCard.getTeam().stringValue();
		System.out.println(
		    name + " of the " + team + " has a batting average of " +
		    batting_average);
		System.out.println("\nDecoded PDU object...");
		System.out.println(decodedCard);
	    }
	    catch (DecodeFailedException e) {
		System.out.println("Decoding failed with error code: " + e);
	    }
	    catch (DecodeNotSupportedException e) {
		System.out.println("Decoding failed with error code: " + e);
	    }
	}
	catch (EncodeFailedException e) {
	    System.out.println("Encoding failed with error code: " + e);
	}
	catch (EncodeNotSupportedException e) {
	    System.out.println("Encoding failed with error code: " + e);
	}

		//
		// Encode the PDU object again to a file.
		//
	System.out.println("Encoding the PDU object to a file and decoding from the file...");
	try {
	    Coder coder = Baseball.getBERCoder();
	    FileOutputStream sink = new FileOutputStream("tbcas.ber");

		//
		// Enable tracing information from the encoder and decoder.
		//
	    coder.enableEncoderDebugging();
	    coder.enableDecoderDebugging();

		//
		// Print the unencoded PDU object.
		//
	    System.out.println("\nUnencoded PDU...\n");
	    System.out.println(myCard);

		//
		// Encode the PDU object.
		//
	    System.out.println("\nThe encoder's trace messages...");
	    coder.encode(myCard, sink);

	    sink.close();

		//
		// Decode the encoded PDU from the file.
		//
	    try {
		FileInputStream source = new FileInputStream("tbcas.ber");

		System.out.println("\nThe decoder's trace messages...\n");
		BBCard decodedCard = (BBCard)coder.decode(source, new BBCard());
		source.close();
		System.out.println("PDU decoded.");
			//
			// Print out the player's batting average.
			//
		double batting_average = decodedCard.getBatting_average();
		String name = decodedCard.getName().stringValue();
		String team = decodedCard.getTeam().stringValue();
		System.out.println(
		    name + " of the " + team + " has a batting average of " +
		    batting_average);
		System.out.println("\nDecoded PDU object...");
		System.out.println(decodedCard);
	    }
	    catch (DecodeFailedException e) {
		System.out.println("Decoding failed with error code: " + e);
	    }
	    catch (DecodeNotSupportedException e) {
		System.out.println("Decoding failed with error code: " + e);
	    }
	    catch (IOException e) {
		System.out.println("I/O exception code: " + e);
	    }
	}
	catch (EncodeFailedException e) {
	    System.out.println("Encoding failed with error code: " + e);
	}
	catch (EncodeNotSupportedException e) {
	    System.out.println("Encoding failed with error code: " + e);
	}
	catch (IOException e) {
	    System.out.println("I/O exception code: " + e);
	}
		//
		// Do a final cleanup.
		//
	Baseball.deinitialize();
    }
}
